"""HEDit: Multi-agent system for HED annotation generation."""

from src.version import __version__, __version_info__, get_version, get_version_info

__all__ = ["__version__", "__version_info__", "get_version", "get_version_info"]
